@@ -6,14 +6,84 @@ import string
 from . import PostProcess
 from . import ConvertFix as CF
 from aksharamukha.ScriptMap.EastIndic import PhagsPa
-from aksharamukha.ScriptMap.MainIndic import Tamil, Malayalam, Limbu
+from aksharamukha.ScriptMap.MainIndic import Tamil, Malayalam, Limbu, Chakma
 ### Use escape char in all functions
 
+def ShowChillus(Strng):
+
+    return PostProcess.MalayalamChillu(Strng, True, True)
+
+def SanskritLexicaizeHK(Strng):
+
+    return Strng
+
+def ThaiPhonetic(Strng):
+    Strng = Strng.replace('ด', 'ท')
+    Strng = Strng.replace('บ', 'พ')
+    Strng = Strng.replace('ก\u0325', 'ค')
+    Strng = Strng.replace('จ\u0325', 'ช')
+    Strng = Strng.replace('งํ', 'ง')
+
+    Strng = Strng.replace('\u035C', '')
+
+    Strng = Strng.replace('\u0E47', '')
+
+    Strng += "\u02BB\u02BB"
+
+    return Strng
+
+def SaurastraHaaruColonTamil(Strng):
+    Strng = Strng.replace('ன', 'ந')
+
+    ListVS = '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Tamil'))
+
+    Strng = re.sub('(' + ListVS + ')' + '(:)' , r'\2\1', Strng)
+
+    chars = '([நமரல])'
+
+    Strng = re.sub(chars + ':', r'\1' + '\uA8B4', Strng)
+
+    return Strng
+
+def ChakmaPali(Strng):
+    Strng = Strng.replace('\U00011147', '𑄤') # Replace Ya
+    Strng = Strng.replace('𑄠', '𑄡') # Replace vA
+
+    listC = '('+"|".join(sorted(GM.CrunchSymbols(GM.Consonants,"Chakma")+Chakma.VowelMap[:1],key=len,reverse=True))+')'
+    listV = '('+"|".join(sorted(GM.CrunchSymbols(GM.VowelSigns,"Chakma")+Chakma.ViramaMap+['\U00011133'],key=len,reverse=True))+')'
+
+    Strng = Strng.replace("\u02BD","")
+
+    Strng = Strng.replace('\U00011102', '\U00011127')
+
+    # Introduce vowel Sign A ; Chakma - Inharant vowel is AA
+    Strng = re.sub("("+listC+")"+"(?!"+listV+")",r'\1''\u02BE',Strng)
+    Strng = Strng.replace("\U00011127","")
+    Strng = Strng.replace("\u02BE","\U00011127")
+
+    return Strng
+
+def TakriArchaicKha(Strng):
+
+    return Strng.replace('𑚋', '𑚸')
+
 def UrduShortNotShown(Strng):
     Strng += "\u02BB\u02BB"
 
     return Strng
 
+def AnuChandraEqDeva(Strng):
+
+    return AnuChandraEq(Strng, 'Devanagari')
+
+def AnuChandraEq(Strng, script):
+    Chandrabindu = GM.CrunchList('AyogavahaMap', script)[0]
+    Anusvara = GM.CrunchList('AyogavahaMap', script)[1]
+
+    Strng = Strng.replace(Chandrabindu, Anusvara)
+
+    return Strng
+
 def TamilNumeralSub(Strng):
     ListC = '(' + '[கசடதபஜஸ]' + ')'
     ListV = '(' + '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Tamil')) + ')'
@@ -62,6 +132,50 @@ def egrantamil(Strng):
 def siddhammukta(Strng):
     return Strng
 
+def TaiKuen(Strng):
+    return Strng
+
+def TaiThamLao(Strng):
+    return Strng
+
+def ThaiSajjhayaOrthography(Strng):
+    Script = "Thai"
+
+    #cons = "|".join(GM.CrunchSymbols(GM.Consonants, Script)+GM.CrunchList('VowelMap',Script)[0:1])
+    #EAIO = "".join(GM.CrunchList('VowelSignMap',Script)[9:12]+GM.CrunchList('SinhalaVowelSignMap',Script)[:])
+    ## Reorder dve
+    #Strng = re.sub('([' + EAIO + '])' + '(' + cons  + ')' + '(๎)' + '(' + cons + ')', r'\2\3\1\4', Strng)
+
+    Strng = Strng.replace('ัง', 'ังฺ')
+    Strng = Strng.replace('์', 'ฺ')
+    Strng = Strng.replace('๎', 'ฺ')
+    Strng = Strng.replace('ั', '')
+
+    return Strng
+
+def ThaiSajjhayawithA(Strng):
+    Strng = Strng.replace('ะ', '')
+    Strng = ThaiSajjhayaOrthography(Strng)
+
+    return Strng
+
+def LaoSajhayaOrthography(Strng):
+    Strng = Strng.replace('ັງ', 'ັງ຺')
+
+    Strng = re.sub('([ເໂໄ])(.๎)([ຍຣລວຨຩສຫຬ])', r'\2\1\3', Strng)
+
+    Strng = Strng.replace('໌', '຺')
+    Strng = Strng.replace('๎', '຺')
+    Strng = Strng.replace('ັ', '')
+
+    return Strng
+
+def LaoSajhayaOrthographywithA(Strng):
+    Strng = Strng.replace('ະ', '')
+    Strng = LaoSajhayaOrthography(Strng)
+
+    return Strng
+
 # consider adding an optional NUkta to the post consonantal position
 def RemoveSchwaHindi(Strng):
     VowI = "(" + '|'.join(GM.CrunchSymbols(GM.Vowels,'Devanagari')) + ")"
@@ -260,12 +374,33 @@ def PreProcess(Strng,Source,Target):
         Strng = Strng.lower()
 
     if Source == 'Itrans':
-        AltForm = ['aa','ii','uu','RRi','RRI','LLi','LLI','N^','JN','chh','shh','x','GY','.n','.m','.h','AUM', 'E', 'O']
-        NormForm = ['A','I','U','R^i','R^I','L^i','L^I','~N','~n','Ch','Sh','kSh','j~n','M','M','','OM', '^e', '^o']
+        sOm = 'OM'
+        tOm = 'oM'
+
+        punc =  '(' + '|'.join(["\u005C"+x for x in list(string.punctuation)]+ ['\s']
+                    + [x.replace('.', '\.') for x in GM.CrunchSymbols(GM.Signs,Source)[1:3]]) + ')'
+
+        Strng = re.sub(punc + sOm + punc, r'\1' + tOm + r'\2', Strng)
+        Strng = re.sub('^' + sOm + punc, tOm + r'\1', Strng)
+        Strng = re.sub(punc + sOm + '$', r'\1' + tOm, Strng)
+        Strng = re.sub('^' + sOm + '$', tOm, Strng)
+
+        punc = '(\s)'
+
+        Strng = re.sub(punc + sOm + punc, r'\1' + tOm + r'\2', Strng)
+        Strng = re.sub('^' + sOm + punc, tOm + r'\1', Strng)
+        Strng = re.sub(punc + sOm + '$', r'\1' + tOm, Strng)
+        Strng = re.sub('^' + sOm + '$', tOm, Strng)
+
+
+        AltForm = ['O', 'aa','ii','uu','RRi','RRI','LLi','LLI','N^','JN','chh','shh','x','GY','.n','.m','.h', 'AUM', 'E']
+        NormForm = ['^o', 'A','I','U','R^i','R^I','L^i','L^I','~N','~n','Ch','Sh','kSh','j~n','M','M','','oM', '^e']
 
         for x,y in zip(AltForm,NormForm):
             Strng = Strng.replace(x,y)
 
+        Strng = Strng.replace('OM', 'oM')
+
     if Source == 'IAST':
         Strng = Strng.replace("aï", "a_i")
         Strng = Strng.replace("aü", "a_u")
@@ -274,9 +409,19 @@ def PreProcess(Strng,Source,Target):
         Strng = Strng.replace('a:i', 'a_i')
         Strng = Strng.replace('a:u', 'a_u')
 
-    if Source == "ISO" or Source == "IAST" or Source == "Titus":
+    if Source == "ISO" or Source == "IAST" or Source == "Titus" or "RussianCyrillic":
         Strng = CF.VedicSvarasNonDiacritic(Strng)
 
+    if ('↓' in Strng or '↑' in Strng) and Target in GM.IndicScripts :
+        Strng = Strng.replace('↓', '॒')
+        Strng = Strng.replace('↑↑', '᳚')
+        Strng = Strng.replace('↑', '॑')
+
+    if ('↓' in Strng or '↑' in Strng) and Target in GM.LatinScripts :
+        Strng = Strng.replace('↓', '\\_')
+        Strng = Strng.replace('↑↑', '\\"')
+        Strng = Strng.replace('↑', '\\\'')
+
     if Source == "WarangCiti":
         Strng = Strng.replace('\u200D', '\u00D7')
 
@@ -411,6 +556,40 @@ def normalize(Strng,Source):
 
     #Strng = Strng.replace('', "ຣ\uE00A")
 
+    ## Normalization for Bengali, Tamil, Malayalam and Grantha
+
+    # Bengali o/au
+
+    Strng = Strng.replace('ো', 'ো')
+    Strng = Strng.replace('াে', 'ো')
+
+    Strng = Strng.replace('ৌ', 'ৌ')
+    Strng = Strng.replace('ৗে', 'ৌ')
+
+    # Tamil o, O, au
+
+    Strng = Strng.replace('ொ', 'ொ')
+    Strng = Strng.replace('ாெ', 'ொ')
+
+    Strng = Strng.replace('ோ', 'ோ')
+    Strng = Strng.replace('ாே', 'ோ')
+
+    Strng = Strng.replace('ௌ', 'ௌ')
+    Strng = Strng.replace('ௗெ', 'ௌ')
+
+    # Malayalam
+
+    Strng = Strng.replace('ൊ', 'ൊ')
+    Strng = Strng.replace('ാെ', 'ൊ')
+
+    Strng = Strng.replace('ോ', 'ോ')
+    Strng = Strng.replace('ാേ', 'ോ')
+
+    # Grantha
+
+    Strng = Strng.replace('𑍋', '𑍋')
+    Strng = Strng.replace('𑌾𑍇', '𑍋')
+
     return Strng
 
 # Remove ZWJ/ZWNJ characters
